// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.propertysheet.name;

import java.util.ArrayList;
import java.util.List;
import java.util.Vector;

import org.eclipse.jface.viewers.ICellEditorValidator;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.PropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;

import eu.etaxonomy.cdm.common.CdmUtils;
import eu.etaxonomy.cdm.model.agent.INomenclaturalAuthor;
import eu.etaxonomy.cdm.model.agent.Team;
import eu.etaxonomy.cdm.model.agent.TeamOrPersonBase;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NonViralName;
import eu.etaxonomy.cdm.model.name.ZoologicalName;
import eu.etaxonomy.taxeditor.propertysheet.ICdmBasePropertySource;

/**
 * @author p.ciardelli
 * @created 18.05.2009
 * @version 1.0
 */
public class NomenclaturalAuthorshipPropertySource implements ICdmBasePropertySource {

	// Property unique keys
	public static final String P_ID_EDITABLECACHE = "P_ID_EDITABLECACHE";		
	public static final String P_ID_PROTECT_CACHE = "P_ID_PROTECT_CACHE";
	public static final String P_ID_AUTHORSHIPCACHE = "P_ID_AUTHORSHIPCACHE";
	public static final String P_ID_EXAUTHORSHIPCACHE = "P_ID_EXAUTHORSHIPCACHE";
	public static final String P_ID_BASAUTHORSHIPCACHE = "P_ID_BASAUTHORSHIPCACHE";
	public static final String P_ID_EXBASAUTHORSHIPCACHE = "P_ID_EXBASAUTHORSHIPCACHE";
	public static final String P_ID_PUBLICATIONYEAR = "P_ID_PUBLICATIONYEAR";
	public static final String P_ID_ORIGINAL_PUBLICATIONYEAR = "P_ID_ORIGINAL_PUBLICATIONYEAR";
	
    // Property display keys
	public static final String P_EDITABLECACHE = "Editable Cache";
	public static final String P_PROTECT_CACHE = "Protect Cache from overwriting?";
	public static final String P_AUTHORSHIPCACHE = "Authorship";
	public static final String P_EXAUTHORSHIPCACHE = "Ex-Authorship";
	public static String P_BASAUTHORSHIPCACHE = "Basionym Authorship";
	public static String P_EXBASAUTHORSHIPCACHE = "Ex-Basionym Authorship";
	public static final String P_PUBLICATIONYEAR = "Publication Year";
	public static final String P_ORIGINAL_PUBLICATIONYEAR = "Original Publication Year";	

	private static final int CACHE_NOT_PROTECTED = 0;
	private static final int CACHE_PROTECTED = 1;
	
	private NonViralName name;
	
	public NomenclaturalAuthorshipPropertySource(NonViralName name) {
		this.name = name;

		initDescriptors();
	}
	
	protected void initDescriptors() {
		
		List<String> displayFields = new ArrayList<String>();
			
		// Cache fields
		displayFields.add(P_ID_EDITABLECACHE);
		displayFields.add(P_ID_PROTECT_CACHE);
		
		if (NomenclaturalCode.ICZN.equals(name.getNomenclaturalCode())) {
			displayFields.add(P_ID_PUBLICATIONYEAR);
			displayFields.add(P_ID_ORIGINAL_PUBLICATIONYEAR);
			
			P_BASAUTHORSHIPCACHE = "Original Authorship";
			P_EXBASAUTHORSHIPCACHE = "Ex-Original Authorship";
		}
		
    	displayFields.add(P_ID_AUTHORSHIPCACHE);
    	displayFields.add(P_ID_EXAUTHORSHIPCACHE);
    	displayFields.add(P_ID_BASAUTHORSHIPCACHE);
    	displayFields.add(P_ID_EXBASAUTHORSHIPCACHE);
    
		
		for (String field : displayFields) {
			addDescriptor(field);
		}
	}

	protected Vector<PropertyDescriptor> descriptors = new Vector<PropertyDescriptor>();
	
	protected void addDescriptor(String id) {
				
		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			descriptors.addElement(
					new TextPropertyDescriptor(P_ID_EDITABLECACHE, P_EDITABLECACHE));
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			descriptors.addElement(
					new ComboBoxPropertyDescriptor(P_ID_PROTECT_CACHE, P_PROTECT_CACHE, 
							new String[] {"no", "yes"}));
		}		
		
		boolean isProtectedCache = name.isProtectedAuthorshipCache();

		// Publication year
		if (id.equals(P_ID_PUBLICATIONYEAR)) {
			PropertyDescriptor descriptor = (isProtectedCache ?
					new PropertyDescriptor(P_ID_PUBLICATIONYEAR, P_PUBLICATIONYEAR) :
					new TextPropertyDescriptor(P_ID_PUBLICATIONYEAR, P_PUBLICATIONYEAR));
			descriptor.setValidator(new IntegerValidator());
			descriptors.addElement(descriptor);
		}
		
		// Original publication year
		if (id.equals(P_ID_ORIGINAL_PUBLICATIONYEAR)) {
			PropertyDescriptor descriptor = (isProtectedCache ?
					new PropertyDescriptor(P_ID_ORIGINAL_PUBLICATIONYEAR, P_ORIGINAL_PUBLICATIONYEAR) :
					new TextPropertyDescriptor(P_ID_ORIGINAL_PUBLICATIONYEAR, P_ORIGINAL_PUBLICATIONYEAR));
			descriptor.setValidator(new IntegerValidator());
			descriptors.addElement(descriptor);
		}
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_AUTHORSHIPCACHE, P_AUTHORSHIPCACHE) :
							new AuthorshipSearchDescriptor(P_ID_AUTHORSHIPCACHE, P_AUTHORSHIPCACHE, (TeamOrPersonBase) name.getCombinationAuthorTeam()));
		}
		
		// Ex-Authorship cache
		if (id.equals(P_ID_EXAUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_EXAUTHORSHIPCACHE, P_EXAUTHORSHIPCACHE) :						
							new AuthorshipSearchDescriptor(P_ID_EXAUTHORSHIPCACHE, P_EXAUTHORSHIPCACHE, (TeamOrPersonBase) name.getExCombinationAuthorTeam()));
		}		
		
		// Basionym Authorship cache
		if (id.equals(P_ID_BASAUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?
							new PropertyDescriptor(P_ID_BASAUTHORSHIPCACHE, P_BASAUTHORSHIPCACHE) :
							new AuthorshipSearchDescriptor(P_ID_BASAUTHORSHIPCACHE, P_BASAUTHORSHIPCACHE, (TeamOrPersonBase) name.getBasionymAuthorTeam()));
		}		
		
		// Ex-Basionym Authorship cache
		if (id.equals(P_ID_EXBASAUTHORSHIPCACHE)) {
			descriptors.addElement(
					isProtectedCache ?					
							new PropertyDescriptor(P_ID_EXBASAUTHORSHIPCACHE, P_EXBASAUTHORSHIPCACHE) :
							new AuthorshipSearchDescriptor(P_ID_EXBASAUTHORSHIPCACHE, P_EXBASAUTHORSHIPCACHE, (TeamOrPersonBase) name.getExBasionymAuthorTeam()));
		}		
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getEditableValue()
	 */
	public Object getEditableValue() {
		return CdmUtils.Nz(name.getAuthorshipCache());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyDescriptors()
	 */
	public IPropertyDescriptor[] getPropertyDescriptors() {
		return (IPropertyDescriptor[]) descriptors.toArray(
                new IPropertyDescriptor[descriptors.size()]);
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#getPropertyValue(java.lang.Object)
	 */
	public Object getPropertyValue(Object id) {

		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			return CdmUtils.Nz(name.getAuthorshipCache());
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			return name.isProtectedAuthorshipCache() ? CACHE_PROTECTED : CACHE_NOT_PROTECTED;
		}
		
		// Publication year
        if (id.equals(P_ID_PUBLICATIONYEAR)) {
        	
        	if (((ZoologicalName) name).getPublicationYear() == null) {
        		return "";
        	}
        	return Integer.toString(((ZoologicalName) name).getPublicationYear());
        }

		// Original publication year
        if (id.equals(P_ID_ORIGINAL_PUBLICATIONYEAR)) {
        	
        	if (((ZoologicalName) name).getOriginalPublicationYear() == null) {
        		return "";
        	}
        	return Integer.toString(((ZoologicalName) name).getOriginalPublicationYear());
        }
        
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			if (name.getCombinationAuthorTeam() != null) {
				return CdmUtils.Nz(name.getCombinationAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}
		
		// Ex-Authorship cache
		if (id.equals(P_ID_EXAUTHORSHIPCACHE)) {
			if (name.getExCombinationAuthorTeam() != null) {
				return CdmUtils.Nz(name.getExCombinationAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}		
		
		// Basionym Authorship cache
		if (id.equals(P_ID_BASAUTHORSHIPCACHE)) {
			if (name.getBasionymAuthorTeam() != null) {
				return CdmUtils.Nz(name.getBasionymAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}		
		
		// Ex-Basionym Authorship cache
		if (id.equals(P_ID_EXBASAUTHORSHIPCACHE)) {
			if (name.getExBasionymAuthorTeam() != null) {
				return CdmUtils.Nz(name.getExBasionymAuthorTeam().getNomenclaturalTitle());
			} else {
				return "";
			}
		}	
		
		return null;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#isPropertySet(java.lang.Object)
	 */
	public boolean isPropertySet(Object id) {
		return false;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#resetPropertyValue(java.lang.Object)
	 */
	public void resetPropertyValue(Object id) {}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.views.properties.IPropertySource#setPropertyValue(java.lang.Object, java.lang.Object)
	 */
	public void setPropertyValue(Object id, Object value) {

		// Editable cache
		if (id.equals(P_ID_EDITABLECACHE)) {
			name.setAuthorshipCache((String) value);
			name.generateAuthorship();
		}
		
		// Protect cache?
		if (id.equals(P_ID_PROTECT_CACHE)) {
			name.setProtectedAuthorshipCache(
					((Integer) value).intValue() == CACHE_PROTECTED);
		}
		
		// Publication year
		if (id.equals(P_ID_PUBLICATIONYEAR)) {
			try {
				((ZoologicalName) name).setPublicationYear(yearToInteger((String) value));
			} catch (NumberFormatException e) {
				// Do nothing - validator should prevent this
			}
        }
		
		// Original publication year
		if (id.equals(P_ID_ORIGINAL_PUBLICATIONYEAR)) {
			try {
				((ZoologicalName) name).setOriginalPublicationYear(yearToInteger((String) value));
			} catch (NumberFormatException e) {
				// Do nothing - validator should prevent this
			}
        }
		
		// Authorship cache
		if (id.equals(P_ID_AUTHORSHIPCACHE)) {
			name.setCombinationAuthorTeam((INomenclaturalAuthor) value);
		}
		
		// Ex-Authorship cache
		if (id.equals(P_ID_EXAUTHORSHIPCACHE)) {
			name.setExCombinationAuthorTeam((INomenclaturalAuthor) value);
		}		
		
		// Basionym Authorship cache
		if (id.equals(P_ID_BASAUTHORSHIPCACHE)) {
			name.setBasionymAuthorTeam((INomenclaturalAuthor) value);
		}		
		
		// Ex-Basionym Authorship cache
		if (id.equals(P_ID_EXBASAUTHORSHIPCACHE)) {
			name.setExBasionymAuthorTeam((INomenclaturalAuthor) value);
		}	
	}

	private Integer yearToInteger(String year) throws NumberFormatException {
		if (year.length() == 0) {
			return null;
		}
		return Integer.parseInt(year);
	}
	
	class IntegerValidator implements ICellEditorValidator {

		public String isValid(Object value) {
			String msg = null;
			try {
				yearToInteger((String) value);
			} catch (NumberFormatException e) {
				msg = "Year contains non-integer characters.";
			}
			return msg;
		}
	}
}