// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.navigation.navigator;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.UUID;

import org.apache.log4j.Logger;
import org.eclipse.core.runtime.IAdaptable;
import org.eclipse.jface.viewers.TreePath;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.IMemento;
import org.eclipse.ui.IViewSite;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.navigator.CommonNavigator;

import eu.etaxonomy.cdm.api.conversation.ConversationHolder;
import eu.etaxonomy.cdm.api.conversation.IConversationEnabled;
import eu.etaxonomy.cdm.model.common.CdmBase;
import eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap;
import eu.etaxonomy.taxeditor.model.DataChangeBridge;
import eu.etaxonomy.taxeditor.model.IDataChangeBehavior;
import eu.etaxonomy.taxeditor.store.CdmStore;

/** 
 * Taxonomic tree implementation using Common Navigator Framework.
 * 
 * @author p.ciardelli
 * @created 02.06.2009
 * @version 1.0
 */
public class TaxonNavigator extends CommonNavigator implements IConversationEnabled {
	private static final Logger logger = Logger
			.getLogger(TaxonNavigator.class);

	public static final String ID = "eu.etaxonomy.taxeditor.navigation.navigator"; //$NON-NLS-1$

	private static final String TREE_PATHS = "treepaths";
	private static final String TREE_PATH = "treepath";

	private ConversationHolder conversation;

	private IDataChangeBehavior dataChangeBehavior;

	private IMemento memento;
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#getInitialInput()
	 */
	@Override
	protected IAdaptable getInitialInput() {
		conversation = CdmStore.NewTransactionalConversation();
		conversation.registerForDataStoreChanges(this);
		this.getCommonViewer().refresh();
		return new Root();
	}
	
	public void init(IViewSite site, IMemento memento)
			throws PartInitException {
		super.init(site, memento);
		
		this.memento = memento;
	}
	
	/**
	 * Restores state from a tree path memento using space-separated
	 * Taxon UUIDs.
	 * 
	 * @param memento
	 */
	private void restoreState(IMemento memento) {
		if (memento == null) {
			return;
		}
		Set<TreePath> treePaths = new HashSet<TreePath>();
		memento = memento.getChild(TREE_PATHS);
		if (memento != null) {
			IMemento[] treePathMementos = memento.getChildren(TREE_PATH);
			if (treePathMementos.length > 0) {
				for (IMemento treePathMemento : treePathMementos) {
					TreePath treePath = createTreePathFromString(treePathMemento.getID());
					if (treePath != null) {
						treePaths.add(treePath);
					}
				}
			}
		}
		if (treePaths.size() > 0) {
			this.getCommonViewer().setExpandedTreePaths(treePaths.toArray(new TreePath[0]));
		}
	}

	/**
	 * @param string
	 * @return
	 */
	private TreePath createTreePathFromString(String string) {

		List<CdmBase> pathList = new ArrayList<CdmBase>();
		
		if(string.length() == 0) return null; 
		
		for (String uuid : string.split(" ")) {
			CdmBase cdmBaseObject = CdmStore.getTaxonService().getTaxonNodeByUuid(UUID.fromString(uuid));
			if (cdmBaseObject == null) {
				// is this a tree uuid?
				cdmBaseObject = CdmStore.getTaxonService().getTaxonomicTreeByUuid(UUID.fromString(uuid));
				
				if(cdmBaseObject == null) return null;
			}
			pathList.add(cdmBaseObject);
		}
		return new TreePath(pathList.toArray());
	}

	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#saveState(org.eclipse.ui.IMemento)
	 */
	public void saveState(IMemento memento) {

		super.saveState(memento);
		
		memento = memento.createChild(TREE_PATHS);
		TreePath[] treePaths = this.getCommonViewer().getExpandedTreePaths();
		for (TreePath treePath : treePaths) {
			int pathLength = treePath.getSegmentCount();
			String path = "";
			for (int i = 0; i < pathLength; i++) {
				Object segment = treePath.getSegment(i);
				if (segment instanceof CdmBase) {
					path += ((CdmBase) segment).getUuid().toString() + " ";
				} else {
					logger.warn("Non-taxon tree path segment " + segment);
				}
			}
			logger.info("Creating memento child for path " + path.trim());
			memento.createChild(TREE_PATH, path.trim());
		}
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#createPartControl(org.eclipse.swt.widgets.Composite)
	 */
	public void createPartControl(Composite aParent) {
		super.createPartControl(aParent);
		restoreState(memento);		
	}
	

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.api.conversation.IConversationEnabled#getConversationHolder()
	 */
	public ConversationHolder getConversationHolder() {
		return conversation;
	}

	/* (non-Javadoc)
	 * @see eu.etaxonomy.cdm.persistence.hibernate.ICdmPostDataChangeObserver#update(eu.etaxonomy.cdm.persistence.hibernate.CdmDataChangeMap)
	 */
	public void update(CdmDataChangeMap changeEvents) {
		if(dataChangeBehavior == null){
			dataChangeBehavior = new TaxonNavigatorDataChangeBehavior(this);
		}
		
		DataChangeBridge.handleDataChange(changeEvents, dataChangeBehavior);
	}
	
	protected String getFrameToolTipText(Object element) {
		if (element instanceof Root) {
			return "Taxonomic Tree";
		}
		return super.getFrameToolTipText(element);
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.part.WorkbenchPart#dispose()
	 */
	@Override
	public void dispose() {
		conversation.commit();
		conversation.unregisterForDataStoreChanges(this);
		
		super.dispose();
	}
	
	/* (non-Javadoc)
	 * @see org.eclipse.ui.navigator.CommonNavigator#setFocus()
	 */
	@Override
	public void setFocus() {
		super.setFocus();
		getConversationHolder().bind();
	}
}