// $Id$
/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.model;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;

import eu.etaxonomy.cdm.model.description.DescriptionBase;
import eu.etaxonomy.cdm.model.description.DescriptionElementBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.TaxonDescription;
import eu.etaxonomy.cdm.model.description.TextData;
import eu.etaxonomy.cdm.model.media.ImageFile;
import eu.etaxonomy.cdm.model.media.Media;
import eu.etaxonomy.cdm.model.media.MediaRepresentation;
import eu.etaxonomy.cdm.model.media.MediaRepresentationPart;
import eu.etaxonomy.cdm.model.media.ReferencedMedia;
import eu.etaxonomy.cdm.model.taxon.Taxon;

/**
 * @author p.ciardelli
 * @created 31.03.2009
 * @version 1.0
 */
public class ImagesHelper {
	private static final Logger logger = Logger
			.getLogger(ImagesHelper.class);
	
	/**
	 * Quick and dirty method to get an element's first image file.
	 * 
	 * @param element
	 * @return
	 * @deprecated
	 */
	public static ImageFile getImage(DescriptionElementBase element) {
		List<Media> medias = element.getMedia();
		
		for(Media media : medias){
			Set<MediaRepresentation> representations = media.getRepresentations();
			
			for(MediaRepresentation representation : representations){
				List<MediaRepresentationPart> parts = representation.getParts();
				
				for (MediaRepresentationPart part : parts){
					if(part instanceof ImageFile){
						return (ImageFile) part;
					}
				}
			}							
		}
		return null;
	}
	
	public static List<ImageFile> getOrderedImages(DescriptionElementBase element){
		List<ImageFile> imageList = new ArrayList<ImageFile>();
		MediaRepresentation representation = getImageMediaRepresentation(element);
		if (representation != null) {
			for (MediaRepresentationPart part : representation.getParts()){
				if(!(part instanceof ImageFile)){
					throw new RuntimeException("Your database contains media that mix Image Files with non-Image Files.");
				} else {
					imageList.add((ImageFile) part);
				}
			}
		}
		return imageList;
	}
		
	/**
	 * Returns the first Representation with images. If none is found, a
	 * Representation for storing images is created and returned.
	 * 
	 * @param element
	 * @return
	 */
	private static MediaRepresentation getImageMediaRepresentation(DescriptionElementBase element) {
		// Drill down until a representation with images is found
		for(Media media : element.getMedia()){
			Set<MediaRepresentation> representations = media.getRepresentations();
			for(MediaRepresentation representation : representations){
				List<MediaRepresentationPart> parts = representation.getParts();
				for (MediaRepresentationPart part : parts){
					if(part instanceof ImageFile){
						return representation;
					}
				}
			}							
		}		
		// No representation with images found - create
		MediaRepresentation representation = MediaRepresentation.NewInstance();
		Media media = ReferencedMedia.NewInstance();
		element.addMedia(media);
		media.addRepresentation(representation);
		return representation;
	}
	
	/**
	 * @param description
	 * @return
	 * @deprecated
	 */
	public static Set<ImageFile> getImages(TaxonDescription description){
		Set<ImageFile> images = new HashSet<ImageFile>();
		
		for (DescriptionElementBase element : description.getElements()){
			
			Feature feature = element.getFeature();
			
			if(feature.equals(Feature.IMAGE())){
				List<Media> medias = element.getMedia();
				
				for(Media media : medias){
					Set<MediaRepresentation> representations = media.getRepresentations();
					
					for(MediaRepresentation representation : representations){
						List<MediaRepresentationPart> parts = representation.getParts();
						
						for (MediaRepresentationPart part : parts){
							if(part instanceof ImageFile){
								images.add((ImageFile) part);
							}
						}
					}							
				}										
			}
		}
		
		return images;
	}

	/**
	 * @param taxon
	 * @param imageFile
	 */
	public static void addTaxonImage(Taxon taxon, DescriptionBase<?> imageGallery, ImageFile imageFile) {
				
		imageGallery.addElement(createImageElement(imageFile));
		
	}
	
	public static DescriptionElementBase createImageElement(ImageFile imageFile) {
		
		DescriptionElementBase descriptionElement = TextData.NewInstance(Feature.IMAGE());
		
		Media media = Media.NewInstance();
		MediaRepresentation representation = MediaRepresentation.NewInstance();
		
		representation.addRepresentationPart(imageFile);
		
		media.addRepresentation(representation);
		
		descriptionElement.addMedia(media);
		
		return descriptionElement;
		
	}
	
	/**
	 * Adds a new, empty image file to the end of a description element's
	 * ordered list of images.
	 * 
	 * @param element
	 * @return 
	 */
	public static ImageFile addImagePart(DescriptionElementBase element) {
		ImageFile imageFile = ImageFile.NewInstance(null, null);
		getImageMediaRepresentation(element).addRepresentationPart(imageFile);
		return imageFile;
	}
	
	/**
	 * @param taxon
	 * @param imageFile
	 * @deprecated
	 */
	public static void removeTaxonImage(Taxon taxon, DescriptionBase<?> imageGallery, ImageFile imageFile) {
		Set<DescriptionElementBase> elementsToRemove = new HashSet<DescriptionElementBase>();
		
		Set<DescriptionElementBase> images = imageGallery.getElements();
		
		for(DescriptionElementBase descriptionElement : images){
			for(Media media : descriptionElement.getMedia()){
				for(MediaRepresentation representation : media.getRepresentations()){
					for(MediaRepresentationPart part : representation.getParts()){
						if(part.equals(imageFile)){
							elementsToRemove.add(descriptionElement);
						}
					}
				}
			}
		}	
		
		for(DescriptionElementBase descriptionElement : elementsToRemove){
			imageGallery.removeElement(descriptionElement);
		}		
	}

	public static final int UP = 1;
	public static final int DOWN = -1;
	
	/**
	 * @param input
	 * @param selectedImage
	 * @param direction
	 */
	public static void moveImage(DescriptionElementBase element, ImageFile image,
			int direction) {
		
		MediaRepresentation representation = getImageMediaRepresentation(element);
		int index = representation.getParts().indexOf(image);
		if (index == -1) {
			return;
		}		
		if (direction == UP && index != 0) {
			Collections.swap(representation.getParts(), index, index - 1);
		}
		if (direction == DOWN && index < representation.getParts().size() - 1) {
			Collections.swap(representation.getParts(), index, index + 1);
		}
	}

	/**
	 * @param input
	 * @param selectedImage
	 */
	public static void removeImage(DescriptionElementBase element, ImageFile image) {
		getImageMediaRepresentation(element).getParts().remove(image);
	}

	/**
	 * Iterate through all taxon's image galleries until the descriptive element containing
	 * the ImageFile is found.
	 * 
	 * @param image
	 * @param taxon 
	 * @return 
	 */
	public static DescriptionElementBase findImageElement(ImageFile image, Taxon taxon) {
		if (taxon == null) {
			return null;
		}
		for (TaxonDescription description : taxon.getDescriptions()) {
			if (description.isImageGallery()) {
				for (DescriptionElementBase element : description.getElements()) {
					if (getOrderedImages(element).contains(image)) {
						return element;
					}
				}
			}
		}
		return null;		
	}
}