/**
* Copyright (C) 2007 EDIT
* European Distributed Institute of Taxonomy 
* http://www.e-taxonomy.eu
* 
* The contents of this file are subject to the Mozilla Public License Version 1.1
* See LICENSE.TXT at the top of this package for the full license terms.
*/

package eu.etaxonomy.taxeditor.preference;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.apache.log4j.Logger;
import org.eclipse.jface.preference.IPreferenceStore;

import eu.etaxonomy.cdm.api.service.config.ITaxonServiceConfigurator;
import eu.etaxonomy.cdm.api.service.config.impl.TaxonServiceConfiguratorImpl;
import eu.etaxonomy.cdm.hibernate.HibernateProxyHelper;
import eu.etaxonomy.cdm.model.common.DefinedTermBase;
import eu.etaxonomy.cdm.model.common.ExtensionType;
import eu.etaxonomy.cdm.model.common.ICdmBase;
import eu.etaxonomy.cdm.model.common.IDefinedTerm;
import eu.etaxonomy.cdm.model.common.Language;
import eu.etaxonomy.cdm.model.common.MarkerType;
import eu.etaxonomy.cdm.model.common.TermBase;
import eu.etaxonomy.cdm.model.description.Feature;
import eu.etaxonomy.cdm.model.description.PresenceAbsenceTermBase;
import eu.etaxonomy.cdm.model.name.NameRelationshipType;
import eu.etaxonomy.cdm.model.name.NomenclaturalCode;
import eu.etaxonomy.cdm.model.name.NomenclaturalStatusType;
import eu.etaxonomy.cdm.model.name.Rank;
import eu.etaxonomy.cdm.model.name.SpecimenTypeDesignationStatus;
import eu.etaxonomy.cdm.model.taxon.TaxonRelationshipType;
import eu.etaxonomy.cdm.persistence.query.MatchMode;
import eu.etaxonomy.taxeditor.model.NomenclaturalCodeHelper;
import eu.etaxonomy.taxeditor.store.VocabularyStore;
import eu.etaxonomy.taxeditor.store.internal.TaxeditorStorePlugin;

/**
 * @author p.ciardelli
 * @created 05.12.2008
 * @version 1.0
 * @author n.hoffmann
 */
public class PreferencesUtil {
	private static final Logger logger = Logger
			.getLogger(PreferencesUtil.class);

	public static final String PREFERRED_NOMENCLATURAL_CODE_KEY = "code";
	
	public static final String TAXON_SERVICE_CONFIGURATOR_TAXA = "taxonServiceConfigurator.doTaxa";
	public static final String TAXON_SERVICE_CONFIGURATOR_SYNONYMS = "taxonServiceConfigurator.doSynonyms";
	public static final String TAXON_SERVICE_CONFIGURATOR_NAMES = "taxonServiceConfigurator.doNamesWithoutTaxa";
	public static final String TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES = "taxonServiceConfigurator.doTaxaByCommonNames";

	public static final String HIDE_DATASOURCE_CHOOSER = "dataSourceChooser.hide";

	public static final String PREFERRED_TERMS_CHANGE = "preferred_terms";

	public static final String HIDE_BULKEDITOR_INFO = "bulkeditorInfo.hide";
	
	
	
	/**
	 * 
	 * @return
	 */
	public static IPreferenceStore getPreferenceStore() {
		return TaxeditorStorePlugin.getDefault().getPreferenceStore();
	}
		
	/**
	 * 
	 * @param preferredCode
	 */
	public static void setPreferredNomenclaturalCode(NomenclaturalCode preferredCode) {
		getPreferenceStore().setValue(PREFERRED_NOMENCLATURAL_CODE_KEY, getPreferenceKey(preferredCode));
	}
		
	/**
	 * @return
	 */
	public static NomenclaturalCode getPreferredNomenclaturalCode() {
	
		for (NomenclaturalCode code : NomenclaturalCodeHelper.getAllCodes()) {
			String preferredCode = getPreferenceStore().getString(PREFERRED_NOMENCLATURAL_CODE_KEY);
			if (getPreferenceKey(code).equals(preferredCode)) {
				return code;
			}
		}
		return null;
	}
	
	/**
	 * Generic method to get term preferences for a term vocabulary
	 * 
	 * @param <T>
	 * @param initialTerms
	 * @return
	 */
	public static <T extends DefinedTermBase> List<T> getPreferredTerms(Set<T> initialTerms){
		
		// set default for non existent preferences
		defaultTerms(initialTerms);
				
		List<T> preferredTerms = new ArrayList<T>();
		
		for (T term : initialTerms){
			if(getPreferenceStore().getBoolean(getPreferenceKey((ICdmBase) term))){
				preferredTerms.add(term);
			}
		}
		
		return preferredTerms;		
	}
	
	private static <T extends DefinedTermBase> boolean defaultTerms(Set<T> initialTerms){
		// set default for non existen preferences
		for(T term : initialTerms){
			if(getPreferenceStore().getBoolean(term.getClass().getName())){
				break;
			}else{
				getPreferenceStore().setValue(term.getClass().getName(), true);
				setPreferredTerms(new ArrayList(initialTerms), initialTerms);
				break;
			}
		}
		return true;
	}
	
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<SpecimenTypeDesignationStatus> getPreferredSpecimenTypeDesignationStatus() { return getPreferredTerms(VocabularyStore.getSpecimenTypeDesignationStatus());}
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<TaxonRelationshipType> getPreferredTaxonRelationshipTypes() { return getPreferredTerms(VocabularyStore.getTaxonRelationshipTypes());}
	
	/** @return a <code>List</code> containing user preferred <code>Feature</code> terms */
	public static List<Feature> getPreferredFeatures() { return getPreferredTerms(VocabularyStore.getFeatures());}

	/** @return a <code>List</code> containing user preferred <code>Rank</code> terms */
	public static List<Rank> getPreferredRanks() { return getPreferredTerms(VocabularyStore.getRanks());}
	
	/** @return a <code>List</code> containing user preferred  <code>PresenceAbsenceTermBase</code> terms */
	public static List<PresenceAbsenceTermBase<PresenceAbsenceTermBase<?>>> getPreferredPresenceAbsenceTerms(){ return getPreferredTerms(VocabularyStore.getPresenceAbsenceTerms());}
	
	/** @return a <code>List</code> containing user preferred  <code>NomenclaturalStatusType</code> terms */
	public static List<NomenclaturalStatusType> getPreferredNomenclaturalStatusTypes(){	return getPreferredTerms(VocabularyStore.getNomenclaturalStatusTypes());}
	
	/** @return a <code>List</code> containing user preferred  <code>NameRelationshipType</code> terms */
	public static List<NameRelationshipType> getPreferredNameRelationshipTypes(){ return getPreferredTerms(VocabularyStore.getNameRelationshipTypes());}
	
	/** @return a <code>List</code> containing user preferred  <code>Language</code> terms */
	public static List<Language> getPreferredLanguages() { return getPreferredTerms(VocabularyStore.getLanguages()); }
	
	/** @return a <code>List</code> containing user preferred  <code>MarkerType</code> terms */
	public static List<MarkerType> getPreferredMarkerTypes() { return getPreferredTerms(VocabularyStore.getMarkerTypes()); }
	
	/** @return a <code>List</code> containing user preferred  <code>ExtensionType</code> terms */
	public static List<ExtensionType> getPreferredExtensionTypes() { return getPreferredTerms(VocabularyStore.getExtensionTypes()); }
	
	
	/**
	 * Generic method to set term preferences
	 * 
	 * @param preferredTerms
	 * @param initialTerms
	 */
	public static <T extends DefinedTermBase> void setPreferredTerms(List<T> preferredTerms, Set<T> initialTerms){
		for(TermBase term : initialTerms){
			getPreferenceStore().setValue(getPreferenceKey(term), preferredTerms.contains(term));
		}
	}
	
	/**
	 * Construct a unique key using the CdmBase object's uuid
	 * 
	 * @param cdmBase
	 * @return
	 */
	private static String getPreferenceKey(ICdmBase cdmBase) {
		cdmBase = (ICdmBase) HibernateProxyHelper.deproxy(cdmBase);
		
		String key = cdmBase.getClass().getName()
				. concat(".")
				. concat(cdmBase.getUuid().toString());
		if (key.contains("javassist")) {
			logger.info("proxy");
		}
		return key;
	}

	/**
	 * Construct a unique key using the CdmBase object's uuid
	 * 
	 * @param cdmBase
	 * @return
	 */
	private static String getPreferenceKey(IDefinedTerm definedTerm) {
		definedTerm = (IDefinedTerm) HibernateProxyHelper.deproxy(definedTerm);
		String key = definedTerm.getClass().getName()
				. concat(".")
				. concat(definedTerm.getUuid().toString());
		if (key.contains("javassist")) {
			logger.info("proxy");
		}
		return key;
	}
	
	/**
	 * Retrieves search preferences from the preference store
	 * 
	 * @return an <code>ITaxonServiceConfigurator</code> to pass to search methods
	 */
	public static ITaxonServiceConfigurator getSearchConfigurator() {
		ITaxonServiceConfigurator configurator = initializeSearchConfigurator();
		// check for existence of at least one preference variable
		if(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_TAXA)){
			// preferences exist
			configurator.setDoTaxa(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_TAXA));
			configurator.setDoSynonyms(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_SYNONYMS));
			configurator.setDoNamesWithoutTaxa(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_NAMES));
			configurator.setDoTaxaByCommonNames(getPreferenceStore().getBoolean(TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES));			
		}
		
		return configurator;
	}
	
	/**
	 * create new preferences, setting all search options to true
	 * @return 
	 */
	public static ITaxonServiceConfigurator initializeSearchConfigurator(){
		ITaxonServiceConfigurator configurator = TaxonServiceConfiguratorImpl.NewInstance();
		
		configurator.setDoTaxa(true);
		configurator.setDoSynonyms(true);
		configurator.setDoNamesWithoutTaxa(true);
		configurator.setDoTaxaByCommonNames(true);
		
		// DEFAULT VALUES		
		// match mode default only
		configurator.setMatchMode(MatchMode.BEGINNING);
		// i don't know what happens to sec at the moment
		configurator.setSec(null);
		// we set page number and size here as this should always be unlimited
		configurator.setPageNumber(0);
		// TODO currently limit results to 1000
		configurator.setPageSize(1000);
		
		return configurator;
	}
	
	/**
	 * Store search preferences
	 * 
	 * @param configurator
	 */
	public static void setSearchConfigurator(ITaxonServiceConfigurator configurator){
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_TAXA, configurator.isDoTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_SYNONYMS, configurator.isDoSynonyms());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_NAMES, configurator.isDoNamesWithoutTaxa());
		getPreferenceStore().setValue(TAXON_SERVICE_CONFIGURATOR_COMMON_NAMES, configurator.isDoTaxaByCommonNames());
	}

	/**
	 * @param class1
	 */
	public static void firePreferencesChanged(Class clazz) {
		getPreferenceStore().firePropertyChangeEvent(PREFERRED_TERMS_CHANGE, null, clazz);
	}
}
